#include "Entity.hpp"
#include "PixelBuffer.hpp"
#include "PixelShaders.hpp"

Entity::Entity()
{
}

Entity::Entity(Position const &topleft, int hitpoints)
	: topleft(topleft), hitpoints(hitpoints)
{
}

Entity::~Entity()
{
}

void Entity::doStep(
	[[maybe_unused]] float PlayerX,
	[[maybe_unused]] float PlayerY,
	[[maybe_unused]] FloatSeconds const & Frametime,
	[[maybe_unused]] Level &level
)
{
}

Position const& Entity::getTopLeft() const
{
	return topleft;
}

float Entity::getLeft() const
{
	return topleft.X;
}

float Entity::getTop() const
{
	return topleft.Y;
}

int Entity::getHitPoints() const
{
	return hitpoints;
}

float Entity::getHitTime() const
{
    return hittime;
}

bool Entity::step(Position const &PlayerPos, FloatSeconds const & Frametime, Level &level)
{
	if(hittime > 0.0f) {
		hittime = std::max(hittime - Frametime.count(), 0.0f);
	}
	if(hitpoints > 0) {
		doStep(PlayerPos.X, PlayerPos.Y, Frametime, level);
	}
	return ((hitpoints > 0) || (hittime > 0.0f));
};

int Entity::takeDamage(Color color)
{
	if(hitpoints == 0) {
		return hitpoints;
	}
	hitpoints--;
	hittime = HIT_TIME;
	hitcolor = color;
	return hitpoints;
}

void Entity::blit(DynamicPixelBuffer &buf)
{
	if(hittime > 0.0f) {
		HitShader hs{
			getSprite(), (hittime / HIT_TIME), hitcolor, (hitpoints == 0)
		};
		buf.blit_topleft(topleft, hs);
	} else if(hitpoints > 0) {
		buf.blit_topleft(topleft, getSprite());
	}
}
