#include <random>

#include "PixelShaders.hpp"

// Avoiding a static initialization order fiasco...
class RNG {
private:
	std::random_device entropy_source;

public:
	std::default_random_engine rng;

	RNG()
		: rng(entropy_source()) {
	}
} rng;

Color HitShader::peek_unchecked(PixelPoint const &pos) const
{
	auto dist = std::uniform_real_distribution<float>{ percent };
	auto src = input.peek(pos);
	if(src == _t) {
		return _t;
	}
	return (dist(rng.rng) > 0.95f) ? color : (is_final ? _t : src);
}

Color DiagonalFadeoutShader::peek_unchecked(PixelPoint const &pos) const
{
	// (percent * 1) would only correspond to the top-left triangle.
	auto src = input.peek(pos);
	auto edge_right = (input.w() * (percent * 2));
	auto edge_bottom = (input.h() * (percent * 2));
	// y = mx + t

	//   Top right point:             0 = m·[edge_right] + t
	// Bottom left point: [edge_bottom] = m·0            + t

	//              0 = m·[edge_right] + [edge_bottom]
	// -[edge_bottom] = m·[edge_right]
	//              m = -[edge_bottom]/[edge_right]
	auto t = edge_bottom;
	auto m = (-edge_bottom / edge_right);

	auto x_end = ((pos.y - t) / m);

	if((pos.x & 0x1) ^ ((pos.y & 0x1) == 1)) {
		return src;
	}
	return (pos.x <= x_end) ? _0 : src;
}
